#ifndef _GTL_ALGORITHM
#define _GTL_ALGORITHM

namespace gtl
{

//translation
template <class I, class U> void translate_each(I i, I end, U& v)
  { for (; i!=end; ++i) (*i)+=v; }

template <class T, class U> void translate(T& object, U& v)
  { translate_each(object.begin(), object.end(), v); }

// rotation
template <class I, class U> void rotate_each(I i, I end, U phi)
  { for (; i!=end; ++i) *i=i->rotate(phi); }

template <class T, class U> void rotate(T& object, U phi)
  { rotate_each(object.begin(), object.end(), phi); }

template <class I, class U, class V> void rotate_each(I i, I end, U phi, V& v)
  { for (; i!=end; ++i) *i=((*i)-=v).rotate(phi)+v; }

template <class T, class U, class V> void rotate(T& object, U phi, V& v)
  { rotate_each(object.begin(), object.end(), phi, v); }

template <class I, class U> void rotate_each(I i, I end, U& a, U& b)
  { for (; i!=end; ++i) *i=i->rotate(a,b); }

template <class T, class U> void rotate(T& object, U& a, U& b)
  { rotate_each(object.begin(), object.end(), a, b); }

template <class I, class U, class V> void rotate_each(I i, I end, U& a, U& b, V& v)
  { 
    for (; i!=end; ++i) *i=((*i)-v).rotate(a-v,b-v)+v;
  }

template <class T, class U, class V> void rotate(T& object, U& a, U& b, V& v)
  { rotate_each(object.begin(), object.end(), a, b, v); }

// scaling
template <class I, class U> void scale_each(I i, I end, U m)
  { for (; i!=end; ++i) *i=m*(*i); }

template <class T, class U> void scale(T& object, U m)
  { scale_each(object.begin(), object.end(), m); }

template <class I, class U, class V> void scale_each(I i, I end, U m, V& v)
  { for (; i!=end; ++i) *i=m*((*i)-v)+v; }

template <class T, class U, class V> void scale(T& object, U m, V& v)
  { scale_each(object.begin(), object.end(), m, v); }

// stretching
template <class I, class V> void stretch_each(I i, I end, V& v)
  { for (; i!=end; ++i) *i=(*i)+v*(*i)*v*(1/v.abs()-1/(v.abs()*v.abs())); }

template <class T, class V> void stretch(T& object, V& v)
  { stretch_each(object.begin(), object.end(), v); }

template <class I, class V> void stretch_each(I i, I end, V& v, V& w)
  { for (; i!=end; ++i) *i=(*i)+v*(*i-w)*v*(1/v.abs()-1/(v.abs()*v.abs())); }

template <class T, class V> void stretch(T& object, V& v, V& w)
  { stretch_each(object.begin(), object.end(), v, w); }



} // namespace gtl

#endif // _GTL_ALGORITHM
