#ifndef _GTL_LINE
#define _GTL_LINE

#include "vertex_array.H"

namespace gtl
{

template <class T=long double>
class line : public vertex_array<T>
  {
  public:
    line() {}
    line(const line& l)
      {
        this->vertices.resize(l.vertices.size());
        for (int i=0; i<l.vertices.size(); i++) this->vertices[i]=l.vertices[i];
      }
    line(const vertex<T>& a, const vertex<T>& b)
      {
        this->vertices.resize(2);
        this->vertices[0]=a;
        this->vertices[1]=b;
      }
    line(const T ax, const T ay, const T bx, const T by)
      {
        this->vertices.resize(2);
        this->vertices[0]=vertex<T>(ax,ay);
        this->vertices[1]=vertex<T>(bx,by);
      }
    template <class U> U size() { return this->vertices.size();}
    line operator+(const vertex<T>& v) const
      {
        line l=line(*this);
        l.vertices.push_back(v);
        return l;
      }
    line operator+=(const vertex<T>& v)
      { this->vertices.push_back(v); return *this; }

    friend std::ostream& operator<<(std::ostream& os, const line& l)
      { 
        os << "[" ;
        if (l.vertices.size()>0) os << l.vertices[0];
        for (int i=1; i<l.vertices.size(); i++) os << "," << l.vertices[i];
        return os << "]";
      }

  };

} // namespace gtl

#endif // _GTL_LINE
