#include "mycanvasview.h"
#include <qmainwindow.h>
#include <qstatusbar.h>


#include "line.H"

MyCanvasView::MyCanvasView ( QCanvas *canvas, QWidget * parent , const char * name , WFlags f ):
    QCanvasView ( canvas, parent, name, f )
    {
	status = ((QMainWindow*) parent)->statusBar();
	activeopn = nopn;
    }

MyCanvasView::~MyCanvasView ()
    {
    }

void MyCanvasView::contentsMousePressEvent( QMouseEvent* e )
    {
        int x=e->x(), y=e->y();
	vertex<int> v(x,y);
        if (e->button()==LeftButton && 0<=x && 600>x && 0<=y && 600>y)
	{
		switch (activeopn)
		{
			case moveopn:
			case rotateopn:
			case scaleopn:
			case stretchopn:
					for (objit=objlist.begin(); objit!=objlist.end() && false==(*objit)->collide(v); objit++);
					if (objlist.end()!=objit) opnpts.push_back( v );
					break;
			default:	break;
		}
	}
	canvas()->update();
    }

void MyCanvasView::contentsMouseReleaseEvent( QMouseEvent* e )
    {
	if (e->button()==RightButton)
	{
		activeopn = nopn;
		status->message( tr("Ready") );
		if (NULL!=activeobj)
		{
			activeobj->del_last();
			if (false==activeobj->is_complete()) delete activeobj;
			activeobj = NULL;
		}
	}
        int x=e->x(), y=e->y();
	vertex<int> v(x,y);
        if (e->button()==LeftButton && 0<=x && 600>x && 0<=y && 600>y)
	{
		switch (activeopn)
		{
			case newline:
					activeobj->add( v );
					if (false==activeinlist && true==activeobj->is_complete())
					{
						objlist.push_back( activeobj );
						activeinlist = true;
					}
					break;
			case newcircle:
					activeobj->add( v );
					if (true==activeobj->is_complete())
					{
						objlist.push_back( activeobj );
						activeinlist = true;
						activeopn=nopn;
						status->message( tr("Ready") );
						activeobj = NULL;
					}
					break;
			case moveopn:
			case rotateopn:
			case scaleopn:
			case stretchopn:
					if (objlist.end()!=objit)
					{
						activeopn = nopn;
						status->message( tr("Ready") );
					}
					break;
			default:	break;
		}
	}
	canvas()->update();
    }

void MyCanvasView::contentsMouseMoveEvent( QMouseEvent* e )
    {
        int x=e->x(), y=e->y();
	vertex<int> v(x,y);
        if (0<=x and 600>x and 0<=y and 600>y)
	{
		switch (activeopn)
		{
			case newline:	
					activeobj->set_last( v );
					break;
			case newcircle:	
					activeobj->set_last( v );
					break;
			case moveopn:
					if (objlist.end()!=objit)
					{
						(*objit)->move(v-opnpts[0]);
						opnpts[0] = v;
					}
					break;
			case rotateopn:	
					if (objlist.end()!=objit) 
					{
						(*objit)->get_centre();
						(*objit)->rotate(opnpts[0], v);
						opnpts[0] = v;
					}
					break;
			case scaleopn:	
					if (objlist.end()!=objit) 
					{
						vertex<int> c = (*objit)->get_centre();
						long double s = (v-c).abs() / (opnpts[0]-c).abs();
						(*objit)->scale(s);
						opnpts[0] = v;
					}
					break;
			case stretchopn:	
					if (objlist.end()!=objit) 
					{
						(*objit)->get_centre();
						(*objit)->stretch(opnpts[0], v);
						opnpts[0] = v;
					}
					break;
			default:	break;
		}
	}
	canvas()->update();
    }

void MyCanvasView::line()
{
    status->message( tr("Drawing line") );
    activeopn = newline;
    if (false==activeinlist) delete activeobj; else if (NULL!=activeobj) activeobj->del_last();
    activeinlist = false;
    activeobj = new Grobj( linetype, canvas() );
}

void MyCanvasView::circle()
{
    status->message( tr("Drawing circle") );
    activeopn = newcircle;
    if (false==activeinlist) delete activeobj; else if (NULL!=activeobj) activeobj->del_last();
    activeinlist = false;
    activeobj = new Grobj( circletype, canvas() );
}

void MyCanvasView::move()
{
    status->message( tr("Move") );
    activeopn = moveopn;
    opnpts.clear();
    if (false==activeinlist) delete activeobj; else if (NULL!=activeobj) activeobj->del_last();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end();
}

void MyCanvasView::rotate()
{
    status->message( tr("Rotate") );
    activeopn = rotateopn;
    opnpts.clear();
    if (false==activeinlist) delete activeobj; else if (NULL!=activeobj) activeobj->del_last();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end();
}

void MyCanvasView::scale()
{
    status->message( tr("Scale") );
    activeopn = scaleopn;
    opnpts.clear();
    if (false==activeinlist) delete activeobj; else if (NULL!=activeobj) activeobj->del_last();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end();
}

void MyCanvasView::stretch()
{
    status->message( tr("Stretch") );
    activeopn = stretchopn;
    opnpts.clear();
    if (false==activeinlist) delete activeobj; else if (NULL!=activeobj) activeobj->del_last();
    activeinlist = false;
    activeobj = NULL;
    objit = objlist.end();
}

