#ifndef _GTL_VERTEX
#define _GTL_VERTEX

#include <iostream>
#include <cmath>

namespace gtl
{

template <class T=long double>
class vertex
  {
  private:
    T _x, _y;
  public:
    vertex() {}
    vertex(const T x, const T y) : _x(x),_y(y) {}
    vertex(const vertex& v) : _x(v._x),_y(v._y) {}

    T x() { return _x; }
    T y() { return _y; }
    
    long double abs ()
      { return std::sqrt(_x*_x+_y*_y); }

    bool operator==(const vertex& rhs) const
      { return _x==rhs._x && _y==rhs._y; }

    vertex operator+(const vertex& rhs) const
      { return vertex(_x+rhs._x, _y+rhs._y); }

    vertex operator+=(const vertex& rhs)
      { return vertex(_x+=rhs._x, _y+=rhs._y); }

    vertex operator-(const vertex& rhs) const
      { return vertex(_x-rhs._x, _y-rhs._y); }

    vertex operator-=(const vertex& rhs)
      { return vertex(_x-=rhs._x, _y-=rhs._y); }

    T operator*(const vertex& rhs) const
      { return _x * rhs._x + _y * rhs._y; }

    template <class U> vertex operator*(U scalar) const
      { 
        U ux = scalar*_x;
        U uy = scalar*_y;
        return vertex((T) ux, (T) uy);
      }

    template <class U> friend vertex operator*(const U scalar, const vertex& v)
      { 
        return vertex<T>((T) scalar*v._x, (T) scalar*v._y);
      }

    vertex scale (const vertex& a, const vertex& b) const
      { 
        T d = std::sqrt((b._x*b._x+b._y*b._y)/(a._x*a._x+a._y*a._y));
        return vertex(_x*d, _y*d); 
      }

    template <class U> vertex rotate (U phi) const
      { return vertex(_x*cos(phi)-_y*sin(phi), _x*sin(phi)+_y*cos(phi)); }

    vertex rotate (const vertex& a, const vertex& b) const
      { // rotation from direction a to b
        T cos_phi = (a._x*b._x+a._y*b._y)/(a._x*a._x+a._y*a._y);
        T sin_phi = (a._x*b._y-a._y*b._x)/(a._x*a._x+a._y*a._y);
        T d = std::sqrt(std::abs(cos_phi*cos_phi-sin_phi*sin_phi));
        cos_phi /= d;
        sin_phi /= d;
        return vertex(_x*cos_phi-_y*sin_phi, _x*sin_phi+_y*cos_phi); 
      }

    friend std::ostream& operator<<(std::ostream& os, const vertex& v)
      { return os << "(" << v._x << ";" << v._y << ")"; }

  }; // class vertex

} // namespace gtl

#endif // _GTL_VERTEX
